library Plug_HVA;

{
  The 'new' HVA plug-in... with TibEd plug-in support.

}

{ Important note about DLL memory management: ShareMem must be the
  first unit in your library's USES clause AND your project's (select
  Project-View Source) USES clause if your DLL exports any procedures or
  functions that pass strings as parameters or function results. This
  applies to all strings passed to and from your DLL--even those that
  are nested in records and classes. ShareMem is the interface unit to
  the BORLNDMM.DLL shared memory manager, which must be deployed along
  with your DLL. To avoid using BORLNDMM.DLL, pass string information
  using PChar or ShortString parameters. }

uses
  Windows,
  Classes,
  SysUtils,
  ShellAPI,
  TempFile in '..\VXLSe_Src\TempFile.pas';

{$R data.res}
{$R *.RES}

{ TibEd v1.40 API specification (also known as version 1 or 140)
  ==============================================================

  TTibEdit=function (AHandle: THandle; FileName, FriendlyName, Texture: PChar; ForeColor, HotColor: Integer; BufSize: Integer; var ReturnText: PChar): Integer; stdcall;
  //FileName: actual disk filename
  //FriendlyName: Friendly filename (shown in TibEd)
  //Texture: File to a JPG/BMP texture file used in TibEd. Safe to ignore if you want
  //ForeColor: Foreground text color to go with the texture. Safe to ignore
  //HotColor: Text color when hot-tracking with the texture. Safe to ignore
  //BufSize: Size of the ReturnText text buffer. Ignore it if you don't use ReturnText
  //ReturnText: Use this to return information to TibEd. Not yet used, but can be used in the future to extend this API.
  //returns: You should return the length of the ReturnText. (but since ReturnText isn't used yet, always return 0)
  TTibIsValid=function (FileName: PChar): Integer; stdcall;
  //return 0 -> not a file I can edit
  //return 1 -> I can edit this file
  TTibSupportExtension=function (Ext: PChar): Integer; stdcall;
  //return 0 -> don't support it
  //return 1 -> that's me! (TibIsValid could still be called though!)
  //return 2 -> don't know, call TibIsValid function with actual file to identify further
  TTibAPIVer=function (): Integer; stdcall;
  //returns: API version number. Return 140 for current API (TibEd v1.4) implementation. Never return higher numbers than your plug-in supports!
  TTibAlsoNeed=function (FriendlyName: PChar; var ExtraFiles: PChar): Integer; stdcall;
  //use this function to request extra files (for example, if you're the voxel editor,
  //return MyVoxel.HVA in order to also get access to this file)
  TTibGetPlugInName=function (BufferSize: Integer; var NameDescription: PChar): Integer; stdcall;
  //Retrieve the name/description of this editor
}

  function TibAPIVer(): Integer; stdcall; forward;
  function TibSupportExtension(Ext: PChar): Integer; stdcall; forward;
  function TibIsValid(FileName: PChar): Integer; stdcall; forward;
  function TibEdit(AHandle: THandle; FileName, FriendlyName, Texture: PChar; ForeColor, HotColor: Integer; BufSize: Integer; var ReturnText: PChar): Integer; stdcall; forward;
  function TibAlsoNeed(FriendlyName: PChar; var ExtraFiles: PChar): Integer; stdcall; forward;
  function TibGetPlugInName(BufferSize: Integer; var NameDescription: PChar): Integer; stdcall; forward;
  procedure VXLSE_Link_HVA(FileName: PChar); forward;

exports
  TibAPIVer index 1, TibSupportExtension index 2, TibIsValid index 3, TibGetPlugInName index 4,
  TibAlsoNeed index 5, TibEdit index 6, VXLSE_Link_HVA;

function TibGetPlugInName(BufferSize: Integer; var NameDescription: PChar): Integer; stdcall;
var
  S: String;
  i: Integer;
begin
  S:='HVA Editor v1.1';
  for i:=1 to Length(S) do
    if i<=BufferSize then NameDescription[i-1]:=S[i];
  TibGetPlugInName:=Length(S);
end;

//does the voxel actually need it's accompanying hva? Perhaps. But not this version!
function TibAlsoNeed(FriendlyName: PChar; var ExtraFiles: PChar): Integer; stdcall;
var
  S: String;
  i: Integer;
begin
  if CompareStr(ExtractFileExt(FriendlyName),'.hva')=0 then begin
    S:=ChangeFileExt(FriendlyName,'.vxl');
    S:=Copy(S,1,255);
    for i:=1 to Length(S) do
      ExtraFiles[i-1]:=S[i];
    TibAlsoNeed:=Length(S);
  end else
    TibAlsoNeed:=0; //if the extension isn't HVA, then I don't know it either :(
end;

function TibEdit(AHandle: THandle; FileName, FriendlyName, Texture: PChar; ForeColor, HotColor: Integer; BufSize: Integer; var ReturnText: PChar): Integer; stdcall;
var
  S: String;
  Res: TResourceStream;
  StartUp: STARTUPINFO;
  ProcInfo: PROCESS_INFORMATION;
begin
  S:=GiveTempFileName;
  S:=ChangeFileExt(S,'.exe');
  Res := TResourceStream.Create(hInstance,'PROGLITE',RT_RCDATA);
  Res.SaveToFile(S);
  Res.Free;
  StartUp.lpReserved:=nil;
  StartUp.lpDesktop:=nil;
  StartUp.lpTitle:=nil;
  StartUp.dwFlags:=STARTF_USESHOWWINDOW;
  StartUp.wShowWindow:=SW_SHOWMAXIMIZED;
  StartUp.cbReserved2:=0;
  StartUp.lpReserved2:=nil;
  //it works now... but don't ask me how - it's taken me hours just to get this working :)
  if not CreateProcess(PChar(S),PChar(S+' '+FriendlyName),nil,nil,False,NORMAL_PRIORITY_CLASS,nil,PChar(ExtractFilePath(FileName)),StartUp,ProcInfo) then
    raise Exception.Create('CreateProcess error:'+IntToStr(GetLastError));
  if not (ProcInfo.hThread=0) then CloseHandle(ProcInfo.hThread);
  if not (ProcInfo.hProcess=0) then begin
    WaitForSingleObject(ProcInfo.hProcess,INFINITE);
    CloseHandle(ProcInfo.hProcess);
  end;
  DeleteFile(S);
  TibEdit:=0;
end;

function TibAPIVer(): Integer; stdcall;
begin
  TibAPIVer:=140;
end;

function TibSupportExtension(Ext: PChar): Integer; stdcall;
begin
  if CompareStr(LowerCase(Ext),'hva')=0 then  //only take files with the hva extension.
    TibSupportExtension:=2  //perform check
  else
    TibSupportExtension:=0;
end;

function TibIsValid(FileName: PChar): Integer; stdcall;
begin
  TibIsValid:=1; //unsupported
  try
    //I could put validation routines here - but I don't have any (yet)
  except //ignore exceptions
  end;
end;

procedure VXLSE_Link_HVA(FileName: PChar);
var
  S: String;
  Res: TResourceStream;
  StartUp: STARTUPINFO;
  ProcInfo: PROCESS_INFORMATION;
begin
  S:=GiveTempFileName;
  S:=ChangeFileExt(S,'.exe');
  Res := TResourceStream.Create(hInstance,'Prog',RT_RCDATA);
  Res.SaveToFile(S);
  Res.Free;
  StartUp.lpReserved:=nil;
  StartUp.lpDesktop:=nil;
  StartUp.lpTitle:=nil;
  StartUp.dwFlags:=STARTF_USESHOWWINDOW;
  StartUp.wShowWindow:=SW_SHOWMAXIMIZED;
  StartUp.cbReserved2:=0;
  StartUp.lpReserved2:=nil;
  //it works now... but don't ask me how - it's taken me hours just to get this working :)
  if not CreateProcess(PChar(S),PChar(S+' '+ExtractFileName(FileName)),nil,nil,False,NORMAL_PRIORITY_CLASS,nil,PChar(ExtractFilePath(FileName)),StartUp,ProcInfo) then
    raise Exception.Create('CreateProcess error:'+IntToStr(GetLastError));
  if not (ProcInfo.hThread=0) then CloseHandle(ProcInfo.hThread);
  if not (ProcInfo.hProcess=0) then begin
    WaitForSingleObject(ProcInfo.hProcess,INFINITE);
    CloseHandle(ProcInfo.hProcess);
  end;
  DeleteFile(S);
end;

begin
end.
