unit FormHeaderUnit;

interface

uses
  Windows, Forms, StdCtrls, Controls, Classes, ExtCtrls, Voxel, Sysutils,
  Buttons, Dialogs, Grids;

{$INCLUDE Global_Conditionals.inc}

type
  PVoxel = ^TVoxel;
  TFrmHeader = class(TForm)
    pnlHeader: TPanel;
    txtFileType: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    txtUnknown: TEdit;
    txtNumSections: TEdit;
    txtNumSections2: TEdit;
    Label3: TLabel;
    Label4: TLabel;
    txtStartRemap: TEdit;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    txtEndRemap: TEdit;
    pnlSectionHeader: TPanel;
    Label8: TLabel;
    cmbSection: TComboBox;
    txtNumber: TEdit;
    txtUnknown1: TEdit;
    txtUnknown2: TEdit;
    Label9: TLabel;
    txtName: TEdit;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    txtTrailer: TMemo;
    butClose: TSpeedButton;
    Timer: TTimer;
    grdTrans: TStringGrid;
    Label13: TLabel;
    Label14: TLabel;
    lblSize: TLabel;
    Label15: TLabel;
    txtNormalMode: TEdit;
    procedure cmbSectionChange(Sender: TObject);
    procedure butCloseClick(Sender: TObject);
    procedure txtChange(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure grdTransSetEditText(Sender: TObject; ACol, ARow: Integer;
      const Value: String);
  private
    { Private declarations }
    p: PVoxel;
  public
    { Public declarations }
    procedure SetValues(Vox: PVoxel);
  end;

var
  FrmHeader: TFrmHeader;

implementation

{$R *.DFM}

procedure TFrmHeader.SetValues(Vox: PVoxel);
var
  i: Integer;
begin
  p:=Vox;
  txtFileType.Text:=p^.Header.FileType;
  txtUnknown.Text:=IntToStr(p^.Header.Unknown);
  txtNumSections.Text:=IntToStr(p^.Header.NumSections);
  txtNumSections2.Text:=IntToStr(p^.Header.NumSections2);
  txtStartRemap.Text:=IntToStr(p^.Header.StartPaletteRemap);
  txtEndRemap.Text:=IntToStr(p^.Header.EndPaletteRemap);
  cmbSection.Style:=csDropDownList;
  cmbSection.Items.Clear;
  for i:=0 to p^.Header.NumSections - 1 do begin
    cmbSection.Items.Add(p^.Section[i].Name);
  end;
  cmbSection.ItemIndex:=0;
  cmbSectionChange(Self);
end;

procedure TFrmHeader.cmbSectionChange(Sender: TObject);
var
  i,j,k: Integer;
begin
  //now populate those other list boxes...
  i:=cmbSection.ItemIndex;
  txtName.OnChange:=nil;
  txtName.Text:=p^.Section[i].Name;
  txtName.OnChange:=txtChange;
  txtNumber.OnChange:=nil;
  txtNumber.Text:=IntToStr(p^.Section[i].Header.Number);
  txtNumber.OnChange:=txtChange;
  txtUnknown1.OnChange:=nil;
  txtUnknown1.Text:=IntToStr(p^.Section[i].Header.Unknown1);
  txtUnknown1.OnChange:=txtChange;
  txtUnknown2.OnChange:=nil;
  txtUnknown2.Text:=IntToStr(p^.Section[i].Header.Unknown2);
  txtUnknown2.OnChange:=txtChange;
  txtNormalMode.OnChange:=nil;
  txtNormalMode.Text:=IntToStr(p^.Section[i].Tailer.Unknown);
  txtNormalMode.OnChange:=txtChange;
  txtTrailer.Lines.Clear;
  With p^.Section[i].Tailer do begin
    lblSize.Caption:=Format('%dx%dx%d', [XSize,YSize,ZSize]);
//    txtTrailer.Lines.Add(Format('Normals mode: %d', [Unknown]));
//    txtTrailer.Lines.Add(Format('Size:    %dx%dx%d', [XSize,YSize,ZSize]));
//this is file information, doens't interest user.
//    txtTrailer.Lines.Add(Format('SpanStart: %d, End: %d, Data: %d', [SpanStartOfs, SpanEndOfs, SpanDataOfs]));
    txtTrailer.Lines.Add(Format('Scale:   %g', [Det]));
    {txtTrailer.Lines.Add('');
    txtTrailer.Lines.Add('Transformation matrix:');
    for j:=1 to 3 do begin
      txtTrailer.Lines.Add(Format('%g %g %g %g', [Transform[j,1],Transform[j,2],Transform[j,3],Transform[j,4]]));
    end;}
    for j:=1 to 3 do begin
      for k:=1 to 4 do begin
        grdTrans.Cells[k-1,j-1]:=FloatToStr(Transform[j,k]);
      end;
    end;
//    txtTrailer.Lines.Add('');
    for j:=1 to 3 do begin
      txtTrailer.Lines.Add(Format('Scale '+Chr(87+j)+' min: %g', [MinBounds[j]]));
    end;
    for j:=1 to 3 do begin
      txtTrailer.Lines.Add(Format('Scale '+Chr(87+j)+' max: %g', [MaxBounds[j]]));
    end;
  end;
end;

procedure TFrmHeader.butCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TFrmHeader.txtChange(Sender: TObject);
var
  i: Integer;
begin
//  ShowMessage(Sender.ClassName);
  i:=cmbSection.ItemIndex;
  if i>=0 then begin
    try
      p^.Section[i].SetHeaderName(txtName.Text);
      p^.Section[i].Header.Number:=StrToInt(txtNumber.Text);
      p^.Section[i].Header.Unknown1:=StrToInt(txtUnknown1.Text);
      p^.Section[i].Header.Unknown2:=StrToInt(txtUnknown2.Text);
      p^.Section[i].Tailer.Unknown:=StrToInt(txtNormalMode.Text);
    except
      on EConvertError do begin
        i:=0;
      end;
    end;
  end;
end;

procedure TFrmHeader.TimerTimer(Sender: TObject);
var
  i,j: Integer;
begin
  j:=cmbSection.ItemIndex;
  for i:=0 to p^.Header.NumSections - 1 do begin
    cmbSection.Items.Strings[i]:=p^.Section[i].Name;
  end;
  cmbSection.ItemIndex:=j;
end;

procedure TFrmHeader.grdTransSetEditText(Sender: TObject; ACol,
  ARow: Integer; const Value: String);
var
  i,j,k: Integer;
begin
  i:=cmbSection.ItemIndex;
  if i>=0 then begin
    try
      for j:=1 to 3 do begin
        for k:=1 to 4 do begin
          p^.Section[i].Tailer.Transform[j,k]:=StrToFloat(grdTrans.Cells[k-1,j-1]);
        end;
      end;
    except
      on EConvertError do begin
        i:=0;
      end;
    end;
  end;
end;

end.
