library Plug_VXL;

{ Important note about DLL memory management: ShareMem must be the
  first unit in your library's USES clause AND your project's (select
  Project-View Source) USES clause if your DLL exports any procedures or
  functions that pass strings as parameters or function results. This
  applies to all strings passed to and from your DLL--even those that
  are nested in records and classes. ShareMem is the interface unit to
  the BORLNDMM.DLL shared memory manager, which must be deployed along
  with your DLL. To avoid using BORLNDMM.DLL, pass string information
  using PChar or ShortString parameters. }

uses
  SysUtils,
  Classes,
  Forms, Windows,
  FormMainUnit in 'FormMainUnit.pas' {FrmMain},
  Voxel in 'Voxel.pas',
  FormHeaderUnit in 'FormHeaderUnit.pas' {FrmHeader},
  Voxel3D in 'Voxel3D.pas',
  FormNewVxlUnit in 'FormNewVxlUnit.pas' {FrmNew},
  FormNewSectionSizeUnit in 'FormNewSectionSizeUnit.pas' {FrmNewSectionSize},
  LoadForm in 'LoadForm.pas' {LoadFrm},
  FormOptions in 'FormOptions.pas' {OptionsFrm},
  cls_Config in 'cls_Config.pas',
  Constants in 'Constants.pas',
  Global in 'Global.pas';


{ TibEd v1.40 API specification (also known as version 1 or 140) 
  ==============================================================

  TTibEdit=function (AHandle: THandle; FileName, FriendlyName, Texture: PChar; ForeColor, HotColor: Integer; BufSize: Integer; var ReturnText: PChar): Integer; stdcall;
  //FileName: actual disk filename
  //FriendlyName: Friendly filename (shown in TibEd)
  //Texture: File to a JPG/BMP texture file used in TibEd. Safe to ignore if you want
  //ForeColor: Foreground text color to go with the texture. Safe to ignore
  //HotColor: Text color when hot-tracking with the texture. Safe to ignore
  //BufSize: Size of the ReturnText text buffer. Ignore it if you don't use ReturnText
  //ReturnText: Use this to return information to TibEd. Not yet used, but can be used in the future to extend this API.
  //returns: You should return the length of the ReturnText. (but since ReturnText isn't used yet, always return 0)
  TTibIsValid=function (FileName: PChar): Integer; stdcall;
  //return 0 -> not a file I can edit
  //return 1 -> I can edit this file
  TTibSupportExtension=function (Ext: PChar): Integer; stdcall;
  //return 0 -> don't support it
  //return 1 -> that's me! (TibIsValid could still be called though!)
  //return 2 -> don't know, call TibIsValid function with actual file to identify further
  TTibAPIVer=function (): Integer; stdcall;
  //returns: API version number. Return 140 for current API (TibEd v1.4) implementation. Never return higher numbers than your plug-in supports!
  TTibAlsoNeed=function (FriendlyName: PChar; var ExtraFiles: PChar): Integer; stdcall;
  //use this function to request extra files (for example, if you're the voxel editor,
  //return MyVoxel.HVA in order to also get access to this file)
  TTibGetPlugInName=function (BufferSize: Integer; var NameDescription: PChar): Integer; stdcall;
  //Retrieve the name/description of this editor
}

  function TibAPIVer(): Integer; stdcall; forward;
  function TibSupportExtension(Ext: PChar): Integer; stdcall; forward;
  function TibIsValid(FileName: PChar): Integer; stdcall; forward;
  function TibEdit(AHandle: THandle; FileName, FriendlyName, Texture: PChar; ForeColor, HotColor: Integer; BufSize: Integer; var ReturnText: PChar): Integer; stdcall; forward;
  function TibAlsoNeed(FriendlyName: PChar; var ExtraFiles: PChar): Integer; stdcall; forward;
  function TibGetPlugInName(BufferSize: Integer; var NameDescription: PChar): Integer; stdcall; forward;

exports
  TibAPIVer index 1, TibSupportExtension index 2, TibIsValid index 3, TibGetPlugInName index 4,
  TibAlsoNeed index 5, TibEdit index 6;

{$R *.RES}

function TibGetPlugInName(BufferSize: Integer; var NameDescription: PChar): Integer; stdcall;
var
  S: String;
  i: Integer;
begin
  S:='Voxel Editor II v1.4';
  for i:=1 to Length(S) do
    if i<=BufferSize then NameDescription[i-1]:=S[i];
  TibGetPlugInName:=Length(S);
end;

//does the voxel actually need it's accompanying hva? Perhaps. But not this version!
function TibAlsoNeed(FriendlyName: PChar; var ExtraFiles: PChar): Integer; stdcall;
//var
//  S: String;
//  i: Integer;
begin
{  if CompareStr(ExtractFileExt(FriendlyName),'.vxl')=0 then begin
    S:=ChangeFileExt(FriendlyName,'.hva');
    S:=Copy(S,1,255);
    for i:=1 to Length(S) do
      ExtraFiles[i-1]:=S[i];
    TibAlsoNeed:=Length(S);
  end else}
    TibAlsoNeed:=0; //if the extension isn't voxel, then I don't know it either :(
end;

function TibEdit(AHandle: THandle; FileName, FriendlyName, Texture: PChar; ForeColor, HotColor: Integer; BufSize: Integer; var ReturnText: PChar): Integer; stdcall;
var
  frm: TfrmMain;
//  j: Integer;
begin
  Application.Handle:=AHandle;
  frm:=TfrmMain.Create(Application);

//if you get the error: undeclared identifier:
//you didn't set the flag in global_conditionals.inc!!!
  frm.PlugInit(FileName,FriendlyName);

  frm.ShowModal;
  frm.Free;
  //this causes a crash in the plug-in handler (TibEd). Mmmm...
//  ReturnText:='';

{  j:=1;
  j:=9 div (j-1);
  TibEdit:=j;}
  TibEdit:=0;
end;

function TibAPIVer(): Integer; stdcall;
begin
  TibAPIVer:=140;
end;

function TibSupportExtension(Ext: PChar): Integer; stdcall;
begin
  if CompareStr(LowerCase(Ext),'vxl')=0 then  //only take files with the vxl extension. Don't use validation routines
    TibSupportExtension:=2  //perform check
  else
    TibSupportExtension:=0;
end;

function TibIsValid(FileName: PChar): Integer; stdcall;
var
  fs: TFileStream;
  Vox: TVoxelHeader;
begin
  TibIsValid:=0; //unsupported
  try
    fs:=TFileStream.Create(FileName,fmOpenRead);
    //try to open the file exclusively - that's needed!
    fs.Read(Vox,SizeOf(Vox));
    fs.Free;
    if (Vox.BodySize <0) or (Vox.NumSections <0) or (Vox.NumSections2<0) or (Vox.NumSections<>Vox.NumSections2) then begin
      raise Exception.Create('Invalid values in voxel header');
    end;
    if CompareStr(Vox.FileType,'Voxel Animation'+#0)=0 then begin
      TibIsValid:=1;
      Exit;
    end;
  except //ignore exceptions
  end;
end;

begin
end.
